"""
Template tag tests.
"""

from django.conf import settings
from django.contrib.auth.models import User, AnonymousUser
from django.http import HttpRequest
from django import template
from django.test import TestCase

from django_clicky.templatetags.clicky import ClickyException
from django_clicky.tests.utils import TestSettingsManager


class TrackClickyTagTestCase(TestCase):
    """
    Tests for the `self.render_tag` template tag.
    """

    def setUp(self):
        self.settings_manager = TestSettingsManager()

    def tearDown(self):
        self.settings_manager.revert()

    def render_tag(self, context=None):
        if context is None: context = {}
        t = template.Template("{% load clicky %}{% track_clicky %}")
        return t.render(template.Context(context))

    def test_no_id(self):
        self.settings_manager.delete('CLICKY_SITE_ID')
        self.assertRaises(ClickyException, self.render_tag)

    def test_wrong_id(self):
        self.settings_manager.set(CLICKY_SITE_ID='1234567')
        self.assertRaises(ClickyException, self.render_tag)
        self.settings_manager.set(CLICKY_SITE_ID='123456789')
        self.assertRaises(ClickyException, self.render_tag)

    def test_rendering(self):
        self.settings_manager.set(CLICKY_SITE_ID='12345678',
                CLICKY_RENDER_NON_JS_CODE=True)
        r = self.render_tag()
        self.assertTrue('var clicky_site_id = 12345678;' in r, r)
        self.assertTrue('href="http://getclicky.com/12345678"' in r, r)
        self.assertTrue('src="http://in.getclicky.com/12345678ns.gif"' in r,
                r)

    def test_rendering_no_nonjs(self):
        self.settings_manager.set(CLICKY_SITE_ID='12345678',
                CLICKY_RENDER_NON_JS_CODE=False)
        r = self.render_tag()
        self.assertTrue('var clicky_site_id = 12345678;' in r, r)
        self.assertFalse('href="http://getclicky.com/12345678"' in r, r)
        self.assertFalse('src="http://in.getclicky.com/12345678ns.gif"' in r,
                r)

    def test_render_internal_ip(self):
        self.settings_manager.set(CLICKY_SITE_ID='12345678',
                CLICKY_INTERNAL_IPS=['1.1.1.1'])
        req = HttpRequest()
        req.META['REMOTE_ADDR'] = '1.1.1.1'
        r = self.render_tag({'request': req})
        self.assertEqual(r, "")

    def test_render_internal_ip_forwarded(self):
        self.settings_manager.set(CLICKY_SITE_ID='12345678',
                CLICKY_INTERNAL_IPS=['1.1.1.1'])
        req = HttpRequest()
        req.META['HTTP_X_FORWARDED_FOR'] = '1.1.1.1'
        r = self.render_tag({'request': req})
        self.assertEqual(r, "")

    def test_render_not_internal_ip(self):
        self.settings_manager.set(CLICKY_SITE_ID='12345678',
                CLICKY_INTERNAL_IPS=['1.1.1.1'])
        req = HttpRequest()
        req.META['REMOTE_ADDR'] = '2.2.2.2'
        r = self.render_tag({'request': req})
        self.assertTrue('var clicky_site_id = 12345678;' in r, r)

    def test_render_session_user(self):
        installed_apps = set(settings.INSTALLED_APPS)
        installed_apps.add('django.contrib.auth')
        installed_apps.add('django.contrib.contenttypes')
        self.settings_manager.set(
            INSTALLED_APPS=list(installed_apps),
            TEMPLATE_CONTEXT_PROCESSORS=[
                    'django.contrib.auth.context_processors.auth'],
            CLICKY_SITE_ID='12345678',
            CLICKY_INTERNAL_IPS=['1.1.1.1'],
        )
        user = User(username='test')
        r = self.render_tag({'user': user})
        self.assertTrue("clicky_custom.session = {username: 'test'};" in r, r)

    def test_render_session_anon_user(self):
        installed_apps = set(settings.INSTALLED_APPS)
        installed_apps.add('django.contrib.auth')
        installed_apps.add('django.contrib.contenttypes')
        self.settings_manager.set(
            INSTALLED_APPS=list(installed_apps),
            TEMPLATE_CONTEXT_PROCESSORS=[
                    'django.contrib.auth.context_processors.auth'],
            CLICKY_SITE_ID='12345678',
            CLICKY_INTERNAL_IPS=['1.1.1.1'],
        )
        user = AnonymousUser()
        r = self.render_tag({'user': user})
        self.assertFalse("clicky_custom.session = {username:" in r, r)



"""
Testing utilities.
"""

from django.conf import settings
from django.core.management import call_command
from django.db.models import loading
from django.test.simple import run_tests as django_run_tests


def run_tests():
    """
    Use the Django test runner to run the django-clicky tests.
    """
    django_run_tests([], verbosity=1, interactive=True)


class TestSettingsManager(object):
    """
    From: http://www.djangosnippets.org/snippets/1011/

    A class which can modify some Django settings temporarily for a
    test and then revert them to their original values later.

    Automatically handles resyncing the DB if INSTALLED_APPS is
    modified.
    """

    NO_SETTING = ('!', None)

    def __init__(self):
        self._original_settings = {}

    def set(self, **kwargs):
        for k, v in kwargs.iteritems():
            self._original_settings.setdefault(k, getattr(settings, k,
                    self.NO_SETTING))
            setattr(settings, k, v)
        if 'INSTALLED_APPS' in kwargs:
            self.syncdb()

    def delete(self, *args):
        for k in args:
            try:
                self._original_settings.setdefault(k, getattr(settings, k))
                delattr(settings, k)
            except AttributeError:
                pass  # setting did not exist

    def syncdb(self):
        loading.cache.loaded = False
        call_command('syncdb', verbosity=0, interactive=False)

    def revert(self):
        for k,v in self._original_settings.iteritems():
            if v == self.NO_SETTING:
                delattr(settings, k)
            else:
                setattr(settings, k, v)
        if 'INSTALLED_APPS' in self._original_settings:
            self.syncdb()
        self._original_settings = {}



"""
Clicky template tag.
"""

import re

from django import template
from django.conf import settings
from django.template import Node, TemplateSyntaxError


SITE_ID_RE = re.compile(r'^\d{8}$')
USER_PROPERTY_CODE = """
    <script type="text/javascript">
      var clicky_custom = clicky_custom || {};
      if (!clicky_custom.session) {
        clicky_custom.session = {username: '%(username)s'};
      }
    </script>
"""
JS_TRACKING_CODE = """
    <script type="text/javascript">
    var clicky = { log: function(){ return; }, goal: function(){ return; }};
    var clicky_site_id = %(site_id)s;
    (function() {
      var s = document.createElement('script');
      s.type = 'text/javascript';
      s.async = true;
      s.src = ( document.location.protocol == 'https:' ? 'https://static.getclicky.com/js' : 'http://static.getclicky.com/js' );
      ( document.getElementsByTagName('head')[0] || document.getElementsByTagName('body')[0] ).appendChild( s );
    })();
    </script>
"""
NONJS_TRACKING_CODE = """
    <a title="Google Analytics Alternative" href="http://getclicky.com/%(site_id)s"></a>
    <noscript><p><img alt="Clicky" width="1" height="1" src="http://in.getclicky.com/%(site_id)sns.gif" /></p></noscript>
"""


register = template.Library()


def track_clicky(parser, token):
    """
    Clicky tracking template tag.

    Renders Javascript code to track page visits.  You must supply
    your Clicky Site ID (as a string) in the ``CLICKY_SITE_ID``
    setting.
    """
    bits = token.split_contents()
    if len(bits) > 1:
        raise TemplateSyntaxError("'%s' takes no arguments" % bits[0])
    return TrackClickyNode()

class TrackClickyNode(Node):
    def __init__(self):
        try:
            site_id = settings.CLICKY_SITE_ID
        except AttributeError:
            raise ClickyException("CLICKY_SITE_ID setting not found")
        self.site_id = str(site_id)
        if not SITE_ID_RE.search(self.site_id):
            raise ClickyException("CLICKY_SITE_ID setting must be a "
                    "string containing an eight-digit number: %s" % site_id)
        self.render_non_js_code = getattr(settings,
                'CLICKY_RENDER_NON_JS_CODE', True)
        self.internal_ips = getattr(settings, 'CLICKY_INTERNAL_IPS', ())

    def render(self, context):
        try:
            request = context['request']
            remote_ip = request.META.get('HTTP_X_FORWARDED_FOR',
                    request.META.get('REMOTE_ADDR', ''))
            if remote_ip in self.internal_ips:
                return ""
        except KeyError:
            pass
        try:
            user = context['user']
            if user.is_authenticated():
                user_html = USER_PROPERTY_CODE % {'username': user.username}
            else:
                user_html = ""
        except KeyError, AttributeError:
            user_html = ""
        vars = {'site_id': self.site_id}
        tracking_html = JS_TRACKING_CODE % vars
        if self.render_non_js_code:
            non_js_tracking_html = NONJS_TRACKING_CODE % vars
        else:
            non_js_tracking_html = ""
        return "".join([user_html, tracking_html, non_js_tracking_html])

register.tag('track_clicky', track_clicky)


class ClickyException(Exception):
    """
    Indicates an error with the Clicky tracking set-up.

    This exception is silenced in Django templates.
    """

    silent_variable_failure = True



"""
=============================
Clicky integration for Django
=============================

The django-clicky application integrates Clicky_ analytics into a
Django_ project.

.. _Clicky: http://getclicky.com/
.. _Django: http://www.djangoproject.com

Overview
========

Clicky is an online web analytics tool.  It is similar to Google
Analytics in that it provides statistics on who is visiting your website
and what they are doing.  Clicky provides its data in real time and is
designed to be very easy to use.  This Django application provides a
template tag to add the tracking code to HTML templates.


Installation
============

To install django-clicky, simply place the ``django_clicky``
package somewhere on the Python path.  The application is configured in
the project ``settings.py`` file.  In order to use the template tags,
the ``django_clicky`` package must be present in the
``INSTALLED_APPS`` list::

    INSTALLED_APPS = [
        ...
        'django_clicky',
        ...
    ]

You set your Clicky Site ID in the ``CLICKY_SITE_ID`` setting::

    CLICKY_SITE_ID = '12345678'

(You can find the Site ID in the Info tab of the website Preferences
page on your Clicky account.)


Usage
=====

The django-clicky application currently provides one template tags that
tracks visitor clicks.  In order to use the tag in a template, first
load the django-clicky template library by adding
``{% load clicky %}`` at the top.


Tracking visitor clicks
-----------------------

Clicky uses Javascript to track every visitor click. The
``track_clicky`` tag inserts the tracking code in the HTML page.  The
Clicky web pages recommend adding the code directly before the
closing ``</body>`` HTML tag::

        ...
        {% track_clicky %}
    </body>
    </html>

The Javascript code added by the template tag is asynchronous and works
on both plain HTTP and secure HTTPS pages.  It also contains
fallback HTML code that uses ``<a>`` and ``<img>`` tags to track
browsers with Javascript disabled.  If you want to skip this fallback
code (for example, if you want to add it to the HTML head section) you
can use the ``CLICKY_RENDER_NON_JS_CODE`` setting::

    CLICKY_RENDER_NON_JS_CODE = False

Clicky data can be annotated with `custom properties`_.  The most
obviously useful information is whether the visitor is a logged in user.
The ``track_clicky`` template tag will automatically provide Clicky with
the visitor username if a user has logged into Django.

.. _`custom properties`: http://getclicky.com/help/customization

.. note::

    The template tag can only access the visitor username if the
    Django user is present in the template context as the ``user``
    variable.  You must either use a ``RequestContext`` and have the
    ``django.contrib.auth.context_processors.auth`` context processor in
    the ``TEMPLATE_CONTEXT_PROCESSORS`` setting (which is default), or
    add this variable to the context yourself when you render the
    template.


Ignoring internal visitors
--------------------------

Often you do not want to track clicks from your development or internal
IP addresses.  For this reason you can set the ``CLICKY_INTERNAL_IPS``
to a list or tuple of addresses that the template tag will not be
rendered on::

    CLICKY_INTERNAL_IPS = ['192.168.45.2', '192.168.45.5']

If you already use the ``INTERNAL_IPS`` setting, you could set the
clicky internal addreses to this value.  This will be the default from
version 2.0.0 upwards.

.. note::

    The template tag can only access the visitor IP address if the
    HTTP request is present in the template context as the ``request``
    variable.  For this reason, the ``CLICKY_INTERNAL_IPS`` settings
    only works if you add this variable to the context yourself when you
    render the template, or you use the ``RequestContext`` and add the
    ``django.core.context_processors.request`` context processor to the
    ``TEMPLATE_CONTEXT_PROCESSORS`` setting::

        TEMPLATE_CONTEXT_PROCESSORS = [
            ...
            'django.core.context_processors.request',
            ...
        ]


Changelog
=========

1.2.1
    Stopped development.  Use the Clicky module in django-analytical_.

1.2.0
    Automatically track logged in users.

1.1.0
    Added the ``CLICKY_INTERNAL_IPS`` setting.

1.0.0
    Initial release.

.. _django-analytical: http://packages.python.org/django-analytical

"""

__author__ = "Joost Cassee"
__email__ = "joost@cassee.net"
__version__ = "1.2.1"
__copyright__ = "Copyright (C) 2011 Joost Cassee"
__license__ = "MIT License"



from distutils.core import setup, Command

import django_clicky


class TestCommand(Command):
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        import os
        os.environ['DJANGO_SETTINGS_MODULE'] = 'django_clicky.tests.settings'
        from django_clicky.tests.utils import run_tests
        run_tests()


setup(
    name = 'django-clicky',
    version = django_clicky.__version__,
    license = django_clicky.__license__,
    description = 'Clicky analytics for Django projects',
    long_description = django_clicky.__doc__,
    author = django_clicky.__author__,
    author_email = django_clicky.__email__,
    packages = [
        'django_clicky',
        'django_clicky.templatetags',
        'django_clicky.tests',
    ],
    keywords = ['django', 'analytics', 'clicky'],
    classifiers = [
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    platforms = ['any'],
    url = 'http://github.com/jcassee/django-clicky',
    download_url = 'http://github.com/jcassee/django-clicky/archives/master',
    cmdclass = {'test': TestCommand},
)
